// --- [ main.c ] -------------------------------------------------------------
//
// tab = 3
//
// Testprogramm fr die Routinen in vnc.c
// Ist beim Einschalten kein Stick vorhanden, dann wird mit blinkender LED an PD.2 gewartet.
// Wenn ein Stick gefunden, dann schaltet LED an PD.2 auf ON.
//
// Anschlieend Endlosschleife:
//
// ffnet die Datei FILE_051.TXT (ggf. als leere Datei),
// hngt 10x 512 + 1 Byte an Testdaten hinten dran.
// ffnet die Datei FILE_101.TXT, lscht sie,
// und kopiert anschlieend blockweise den Inhalt von Datei FILE_051.TXT
// auf die Datei FILE_101.TXT.
// Wartet dann auf die Entnahme des Sticks (LED an PD.3 ist ON).
// Wenn er wieder eingesteckt wird, dann beginnt die Schleife von vorne.
//
// Die Datei FILE_051.TXT wird bei jedem Rundgang um 5kb grer.
// Ab 65 KB Dateigre ist das Kopieren nicht mehr mglich, da die zugehrigen
// Zhler nur 16bit breit sind.
// Arbeitet bis 19200 Baud - darber gibt es Fehler bei Schreiben von Daten,
// wenn eine bestehende Datei geffnet und Daten angehngt werden sollen
// (eine leere Datei lt sich beschreiben ...)
//
// Der PORTD ist komplett mit LED's bestckt, zeigen Aktivitten an der seriellen
// Schnittstelle und Statusmeldungen
//
// ----------------------------------------------------------------------------
#define EXTERN

#include <avr/io.h>
#include <inttypes.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "rs232.h"
#include "vnc.h"

// fr den Timer wird ein ISR-Intervall gewhlt, aus dem sich der Sekundentakt
//  ableiten lsst: 7.372.800 / 1024 = 7200, 7200 / 240 = 30 -> 33ms
#define PRELOAD (256 - 240)

void error(uint8_t err);

// ----------------------------------------------------------------------------
int main(void)
{
uint8_t i = 0;
uint8_t z;
uint16_t ii;
uint8_t file1, file2;

TCCR0  = (1<<CS02 | 1<< CS00);					// prescale 1024
TIMSK  = (1<<TOIE0);									// enable TIM0_OVF

time[0] = 30;											// 30s
time[1] = 10;											// 10m
time[2] = 20;											// 20h
time[3] = 1;											// 01.
time[4] = 12;											// Dezember
time[5] = 28;											// 2008 - 1980 = 28

asm volatile ("sei");								// fr den Timer erforderlich
PORTD = 0xFC;
DDRD  = 0xFC;											// PD.2 bis PD.7 zum debuggen

i = 5;
while(i)													// etwas Warten bis der Stick bereit ist
	{
	VNC_timeout = 30;
	while(VNC_timeout);
	PORTD ^= 0xFC;										// 4 LED's blinken
	i--;
	}

PORTD =0xFC;											// 4 LED's aus

rs232_init(TEILER_9600);							// zuerst COM initialisieren, damit an den VNC gesendet werden kann
VNC_init();												// umschalten auf Short Command, einstellen der Baud-Raten

VNC_send_cmd(VNC_EMPTY_CMD);						// Testen, ob ein Stick eingelegt ist
															// dazu ein EMPTY COMMAND (nur CR) senden
i = VNC_get_error();
if (!(i))												// wenn kein Fehler, dann 
	{
	LEDPORT &= ~(1<<LED_READY);					// LED auf Dauerlicht
	}

else 														// kein Stick gefunden
	{
	VNC_insert_disk();								// Warteschleife bis zum Einlegen eines Sticks
	}

															// Hier angekommen sollte ein Stick eingelegt sein !!
if (VNC_get_error()) error(9);					// Wenn ein anderer Fehler vorliegt

while (!(VNC_get_error())) VNC_cd(UP);			// Auf die Hauptebene des Sticks wechseln.
															// Solange aufwrts, wie keine Fehlermeldung kommt

file1 = 51;												// source-Dateiname
file2 = 101;											// target-Dateiname (zum Kopieren)

VNC_make_name(file1, FILE);						// Dateinamen aus vorgegebenem String + Ziffer bilden
VNC_file_access(FILE_CLOSE);						// Datei schlieen (falls offen), Fehlermeldungen ignorieren	
VNC_make_name(file2, FILE);						// Dateinamen aus vorgegebenem String + Ziffer bilden
VNC_file_access(FILE_CLOSE);						// Datei schlieen (falls offen), Fehlermeldungen ignorieren

while(1)
	{
	
	i = 0;
	z = ('A');											// 512 Byte Sendepuffer mit Testdaten fllen
	for(ii = 0; ii < 512; ii++)
		{
		if (i == 31)					
			{
			rx_buffer[ii] = CHR_RETURN;			// 31 Zeichen + RETURN = 32 Zeichen
			z++;											// char incrementieren
			i = 0;
			}
		else
			{
			rx_buffer[ii] = z;							
			i++;
			}
		}	


	VNC_make_name(file1, FILE);					// Dateinamen aus vorgegebenem String + Ziffer bilden

//	VNC_make_name(1, DIR);							// Verzeichnisnamen aus vorgegebenem String + Ziffer bilden
	
//	VNC_md();											// Verzeichnis anlegen
//	VNC_cd(DOWN);										// in das Verzeichnis absteigen

//	VNC_file_access(FILE_CLOSE);					// Datei schlieen (falls offen), Fehlermeldungen ignorieren
//	VNC_file_access(FILE_DELETE);

	VNC_get_filesize();								// liefert Fehler bei leerer Datei

// if (VNC_get_error()) error(1);
//	ultoa(VNC_fs.word[0], rx_buffer, 10);		// die Dateigre an den Anfang des Sendepufferes schreiben

	VNC_file_access(FILE_WRITE);					// Datei zum Schreiben ffnen	
//	VNC_seek(pos);										// Die Schreibposition ndern (Standard: Ende der Datei)
//	pos += 32;

	i = 0;
	while(i< 10)										// 10 * 512 Byte schreiben
		{
		VNC_write_data(512, rx_buffer);
		if (VNC_get_error()) error(7);
		i++;
		}
	VNC_write_data(1, rx_buffer);				// 5120 + 1 Byte schreiben
	if (VNC_get_error()) error(3);
	
	VNC_file_access(FILE_CLOSE);					// Datei schlieen
	if (VNC_get_error()) error(9);

	VNC_copy_file(file1, file2);					// die Datei kopieren
	if (VNC_get_error()) error(6);

//	error(15);											// wenn nun alle 4 LED's blinken, dann wurde kein Fehler erkannt

	VNC_remove_disk();								// wartet auf das Entfernen / Wiedereinlegen des Sticks
	
	}
}


// ----------------------------------------------------------------------------
// TIMER0_OVF:  7...MHZ / (1024 * 240) -> 33ms  Zhlt VNC_timeout herunter auf 0
// als Grundlage fr die Zeitzhlung muss ein Takt gewhlt werden, aus dem sich
// der Sekundentakt ableiten lsst: Die ISR wird 30x /Sekunde aufgerufen
ISR(TIMER0_OVF_vect)
{
static uint8_t tick; 

TCNT0 = PRELOAD;										// Preload neu laden

if(VNC_timeout) VNC_timeout--;					// timeout runterzhlen

tick--;
if (!(tick))											// wenn 1 Sekunde um ist (tick = 30)
	{												
	tick = 30;
	time[0]++;
	if (time[0] > 59)
		{
		time[0] = 0;
		time[1]++;
		if (time[1] > 59)
			{
			time[1] = 0;
			time[2]++;
			if (time[3] > 23)
				{
				time[3] = 0;
				time[4]++;						
				if (time[4] > 31)						// nicht entsprechend der Anzahl der Tage
					{										// eines Monats
					time[4] = 1;						// Tag = 01
					time[5]++;							// nur bis zum Monat addieren
					}
				}
			}
		}
	}
}

// ----------------------------------------------------------------------------
void error(uint8_t err)
{
uint8_t i = 0, j= 0;

j = VNC_get_error();
if (j & 0x0F) i = 0x04;
err *= 16;												// die Fehlermeldung auf high-nibble schieben
//j *= 16;

VNC_exit();												// auf 9600 Baud zurckschalten	

while(1)
	{
	if (j)
		{
		VNC_timeout = 3;								// schnell blinken
		LEDPORT ^= (i + j);
		}
	else
		{
		VNC_timeout = 7;								// langsam blinken
		LEDPORT ^= (err);
		}
	while(VNC_timeout);
	}
}


// --- [ eof ] ----------------------------------------------------------------
