// --- [ vnc.c ] --------------------------------------------------------------
//
// tab = 3
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <inttypes.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "rs232.h"
#include "vnc.h"

//#define SYNCH								// zustzliche Synchs einfgen - ohne erkennbaren Effekt

// --- [ lokale Variablen und Funktionen ] ------------------------------------
													// default-Dateiname, nur die 3 Ziffern
													// werden variiert. Lnge immer 13 Zeichen !
char VNC_filename[13] = "FILE_000.TXT";// 8 + 1 + 3 = 12 + '0' = 13
char VNC_dirname[9]   = "DIR__000";		// default-Verzeichnisnamen
uint8_t VNC_error;

// interne Funktionen
void VNC_switch_SCS(void);				 	// Microcontroller-Befehlssatz
void VNC_wait_for_prompt(void);
void VNC_wait_for_data(uint16_t);
void VNC_synch(void);
void VNC_set_baudrate_9600(void);
void VNC_set_baudrate_19200(void);
void VNC_set_baudrate_38400(void);
void VNC_set_baudrate_57600(void);
void VNC_set_baudrate_115200(void);
void VNC_wait_for_rx(void);

// ----------------------------------------------------------------------------
// sendet ein SYNCH ('E') und wartet auf die Antwort 'E' - oder
// auf ein Timeout
// Errorcodes: nur Timeout
void VNC_synch(void)
{
uint8_t i = 10;
while (i)
	{
	put_c(CHR_SYNCH);
	put_c(CHR_RETURN);

	VNC_wait_for_prompt();					// hier wird auf PROMPT und SYNCH geprft 
	if (VNC_error) i--; else i = 0;		// im Fehlerfalle mehrere Versuche ausfhren
	}
}	

// ----------------------------------------------------------------------------
// Die Initialisierung durchfhren, liefert Erfolg ber VNC_error zurck
void VNC_init(void)
{
uint8_t i = 10;

while(i)
	{
	VNC_timeout = VNC_PROMPT_WAIT;		// kleine Warteschleife, damit lange ECS Meldungen
	while(VNC_timeout);						// abgeschlossen sind (wie: BAD COMMAND)
	
	VNC_switch_SCS();							// zuerst den Microcontroller-Befehlssatz anwhlen
													// damit keine Meldung wie BAD COMMAND mehr ankommt
	if (VNC_error) i--; else i = 0;		// i decrementieren - oder auf 0 setzen und beenden
	}

#if VNC_BAUD_RATE == 19200	
if (!(VNC_error)) VNC_set_baudrate_19200();
#elif VNC_BAUD_RATE == 38400	
if (!(VNC_error)) VNC_set_baudrate_38400(); // Baudrate des VNC von default 9600 auf x ndern
#elif VNC_BAUD_RATE == 57200
if (!(VNC_error)) VNC_set_baudrate_57200(); // Baudrate des VNC von default 9600 auf x ndern
#elif VNC_BAUD_RATE == 115200
if (!(VNC_error)) VNC_set_baudrate_115200();// Baudrate des VNC von default 9600 auf x ndern

#endif
}

// ----------------------------------------------------------------------------
// setzt die durch eine Konstante definierte Baudrate
// Fehler werden ber VNC_error signalisiert. 
void VNC_set_baudrate_9600(void)
{
put_c(VNC_SBD);
put_c(CHR_SPACE);
put_c(0x38);									// 9600
put_c(0x41);									// 9600
put_c(0x00);									// 3. Byte immer '0'
put_c(CHR_RETURN);
													// auf die Antwort warten - es kommen 2 Antworten:
VNC_wait_for_prompt();						// die erste mit der alten Baudrate
													// und nach lngerer Pause,
rs232_init(TEILER_9600);					// in der die Baudrate des Microcontrollers angepasst wird
VNC_wait_for_prompt();						// folgt die 2. Antwort
}

// ----------------------------------------------------------------------------
// setzt die durch eine Konstante definierte Baudrate
// Fehler werden ber VNC_error signalisiert. 
#if VNC_BAUD_RATE == 19200
void VNC_set_baudrate_19200(void)
{
put_c(VNC_SBD);
put_c(CHR_SPACE);
put_c(0x9C);									// 38400
put_c(0x80);									// 38400
put_c(0x00);									// 3. Byte immer '0'
put_c(CHR_RETURN);
													// auf die Antwort warten - es kommen 2 Antworten:
VNC_wait_for_prompt();						// die erste mit der alten Baudrate
													// und nach lngerer Pause,
rs232_init(TEILER_19200);					// in der die Baudrate des Microcontrollers angepasst wird
VNC_wait_for_prompt();						// folgt die 2. Antwort
}
#endif


// ----------------------------------------------------------------------------
// setzt die durch eine Konstante definierte Baudrate
// Fehler werden ber VNC_error signalisiert. 
#if VNC_BAUD_RATE == 38400
void VNC_set_baudrate_38400(void)
{
put_c(VNC_SBD);
put_c(CHR_SPACE);
put_c(0x4E);									// 38400
put_c(0xC0);									// 38400
put_c(0x00);									// 3. Byte immer '0'
put_c(CHR_RETURN);
													// auf die Antwort warten - es kommen 2 Antworten:
VNC_wait_for_prompt();						// die erste mit der alten Baudrate
													// und nach lngerer Pause,
rs232_init(TEILER_38400);					// in der die Baudrate des Microcontrollers angepasst wird
VNC_wait_for_prompt();						// folgt die 2. Antwort
}
#endif
// ----------------------------------------------------------------------------
// setzt die durch eine Konstante definierte Baudrate
// Fehler werden ber VNC_error signalisiert. 
#if VNC_BAUD_RATE == 115200
void VNC_set_baudrate_115200(void)
{
put_c(VNC_SBD);
put_c(CHR_SPACE);
put_c(0x1A);									// 115600
put_c(0x00);									// 115600
put_c(0x00);									// 3. Byte immer '0'
put_c(CHR_RETURN);
													// auf die Antwort warten - es kommen 2 Antworten:
VNC_wait_for_prompt();						// die erste mit der alten Baudrate
													// und nach lngerer Pause,
rs232_init(TEILER_115200);					// in der die Baudrate des Microcontrollers angepasst wird
VNC_wait_for_prompt();						// folgt die 2. Antwort
}
#endif

// ----------------------------------------------------------------------------
// exportiert die lokale Variable VNC_error
uint8_t VNC_get_error(void)
{
return(VNC_error);
}

// ----------------------------------------------------------------------------
// Schaltet in den Modus Short_Instruction_Set (Microcontroller-Befehlssatz)
// muss die erste Aktion sein.
void VNC_switch_SCS(void)
{
put_c('S');										// Sendet den String 'SCS'
put_c('C');
put_c('S');
put_c(CHR_RETURN);

VNC_wait_for_prompt();						// auf die Antwort warten
}

// ----------------------------------------------------------------------------
// Sendet ein bergebenes Kommando 
// wenn cmd = 0, dann wird nur ein RETURN gesendet (= EMPTY cOMMAND)
void VNC_send_cmd(uint8_t cmd)
{

#ifdef SYNCH
VNC_synch();
if (VNC_error) return;
#endif

if (cmd) put_c(cmd);							// zuerst das CMD senden, sofern cmd > 0
put_c(CHR_RETURN);							// dann noch ein RETURN senden
	
VNC_wait_for_prompt();						// auf die Antwort warten
}

// ----------------------------------------------------------------------------
// Legt ein Unterverzeichnis an mit dem Namen, der in der Variablen 'VNC_dirname' steht
void VNC_md(void)
{

put_c(VNC_MKD);								// Befehl	
put_c(CHR_SPACE);								// das Leerzeichen
put_s(VNC_dirname);							// den Verzeichnisnamen 'VNC_dirname' senden
put_c(CHR_RETURN);							// dann noch ein RETURN senden	

VNC_wait_for_prompt();
}


// ----------------------------------------------------------------------------
// Lscht das Unterverzeichnis mit dem Namen, der in der Variabeln 'VNC_dirname' steht
void VNC_rd(void)
{

put_c(VNC_DLD);								// Befehl	
put_c(CHR_SPACE);								// das Leerzeichen
put_s(VNC_dirname);							// den Verzeichnisnamen 'VNC_dirname' senden
put_c(CHR_RETURN);							// dann noch ein RETURN senden	

VNC_wait_for_prompt();
}
// ----------------------------------------------------------------------------
// Wechselt in ein Unterverzeichnis mit dem Namen, der in der Variabeln 'VNC_dirname' 
// steht ... oder in das bergeordnete Verzeichis
// Parameter UP   -> wechselt aufwrts in Richtung root ('..')
//           DOWN -> wechselt nach 'VNC_dirname'
void VNC_cd(uint8_t direction)
{

put_c(VNC_CD);									// Instruction	
put_c(CHR_SPACE);								// SPACE
if (direction == UP)
	{
	put_c('.');									// den Verzeichnisnamen '..' senden
	put_c('.');
	}
else
	{
	put_s(VNC_dirname);						// den Verzeichnisnamen aus 'VNC_dirname' senden
	}
put_c(CHR_RETURN);							// dann noch ein RETURN senden	

VNC_wait_for_prompt();
}

// ----------------------------------------------------------------------------
// ffnet eine Datei zum Lesen oder Schreiben oder 
// schliet oder lscht eine Datei.
// Die Parameterwerte entsprechen dem zugehrigen binren Kommando.
// Der Name der Datei steht in der Variablen 'VNC_filename'.
// Es kann immer nur eine einzige Datei geffnet sein !
// Parameter:	- FILE_WRITE
// 				- FILE_READ
// 				- FILE_CLOSE
// 				- FILE_DELETE
void VNC_file_access(uint8_t modus)
{
uint8_t f_time;								// Hilfsvariable fr Filetime

#ifdef SYNCH
VNC_synch();
if (VNC_error) return;
#endif

put_c(modus);									// der Parameter entspricht dem zugehrigen Kommando
put_c(CHR_SPACE);								// das Leerzeichen
put_s(VNC_filename);							// den Dateinamen senden


if (modus == FILE_WRITE)					// Das Datum nur beim ffnen zum Schreiben senden
	{
	put_c(CHR_SPACE);							// ein zustzliches SPACE senden

	f_time  = time[4] /  8;					// Monat >>3
	f_time += time[5] *  2;					// Jahr   <1 addieren
	put_c(f_time);
	
	f_time  = time[4] * 32;					// Monat <<5
	f_time += time[3];						// Tage addieren
	put_c(f_time);
	
	f_time  = time[1] /  8;					// Minuten >>3
	f_time += time[2] *  8;					// Stunden <<3 addieren	
	put_c(f_time);
	
	f_time  = time[1] * 32;					// die ersten 3Bit der Minuten <<5
	f_time += time[0] /  2;					// Sekunden /2 addieren (die Auflsung betrgt 2 Sekunden (5 Bit)!)	
	put_c(f_time);
	}

put_c(CHR_RETURN);							// dann noch ein RETURN senden
	
VNC_wait_for_prompt();

}

// ----------------------------------------------------------------------------
// Schreibt Daten aus einem anzugebenden Array in eine geffnete Datei
// Die Anzahl ist in dieser Funktion auf 64k beschrnkt !
// Es muss genau die angegebene Anzahl von Bytes gesendet werden.
//
// Beim Lesen durch VNC_read_data() wird immer in den rx_buffer gelesen.
// rx_buffer kann direkt wieder zum Schreiben benutzt werden (->Kopieren)

void VNC_write_data(uint16_t cnt, uint8_t *buffer)
{
uint16_t i = 0;
uint8_t *ptr;

#ifdef SYNCH
VNC_synch();
if (VNC_error) return;
#endif

ptr = &buffer[0];
put_c(VNC_WRF);								// Instruction
put_c(CHR_SPACE);								// space
put_c(0);										// Anzahl Bytes 1/4 (hchstwertiges Byte)
put_c(0);										// Anzahl Bytes 2/4
put_c((uint8_t) (cnt/256));				// Anzahl Bytes 3/4
put_c((uint8_t) cnt);						// Anzahl Bytes 4/4 (niedrigstwertiges Byte)

put_c(CHR_RETURN);							// dann noch ein RETURN senden

while(i < cnt) put_c(ptr[i++]);

VNC_wait_for_prompt();
}

// ---------------------------------------------------------------------------
// liest cnt-Bytes aus der bereits zum Lesen geffneten Datei 'VNC_filename'
// hier beschrnkt auf 64k zu lesende Bytes 
// schreibt die Daten in rx_buffer[]

void VNC_read_data(uint16_t cnt)
{

#ifdef SYNCH
VNC_synch();
if (VNC_error) return;
#endif

put_c(VNC_RDF);								// Instruction
put_c(CHR_SPACE);								// Space

put_c(0);										// Anzahl Bytes 1/4 (hchstwertiges Byte)
put_c(0);										// Anzahl Bytes 2/4
put_c((uint8_t) (cnt/256));				// Anzahl Bytes 3/4
put_c((uint8_t) cnt);						// Anzahl Bytes 4/4 (niedrigstwertiges Byte)
put_c(CHR_RETURN);							// dann noch ein RETURN senden
	
VNC_wait_for_data(cnt);						// wartet auf die angeforderte Anzahl von Bytes
}

// ----------------------------------------------------------------------------
// Holt die Dateigre der Datei 'VNC_filename' und schreibt sie in die globale
// Variable 'VNC_fs'.
// Auf die Union 'VNC_fs' kann via dword, word oder byte zugegriffen werden:
// VNC_fs.dword, VNC_fs.word[0..1], VNC_fs.byte[0..3]

void VNC_get_filesize(void)
{

VNC_error = VNC_ERR_TO;						// Timeout error als Vorgabe
VNC_fs.word[0] = 0;							// lschen
VNC_fs.word[1] = 0;
uint8_t i = 0, j = 0;
uint8_t buffer[7];							// zum Einlesen der Dateigre und der folgenden 3 Zeichen

#ifdef SYNCH
VNC_synch();
if (VNC_error) return;
#endif

put_c(VNC_DIR);								// Instruction senden
put_c(CHR_SPACE);								// Space senden
put_s(VNC_filename);							// der Name der Datei steht in VNC_filename
put_c(CHR_RETURN);							// dann noch ein RETURN senden:

// Antwort: CR + DATEINAME + CR + BYTE1 + BYTE2+ BYTE3 + BYTE4 + CR + '>' + CR
// den RX_Interrupt disablen, solange auf eine Antwort gewartet wird
UCSRB &= ~(1<<RXCIE);

while(VNC_filename[j]) j++;				// die Lnge des Dateinamens bestimmen
j += 2;											// 2 addieren fr fhrendes und folgendes CR

while (i < j)									// CR + Dateiname + CR empfangen
	{
	VNC_wait_for_rx();		
	if (!(VNC_timeout)) break;				// bei timeout beenden
	i++;
	buffer[0] = UDR;
	}

i = 0;											// nun kommen 4 Byte Daten mit der Dateigre
while (i < 7)
	{
	VNC_wait_for_rx();		
	if (!(VNC_timeout)) break;				// bei timeout beenden
   buffer[i++] = UDR;						// Inhalt von UDR speichern
	}

buffer[5] += buffer[6];	
if ((VNC_timeout) && (buffer[5] == (CHR_PROMPT + CHR_RETURN)))
	{
	VNC_fs.byte[0] = buffer[2];			// low und high byte jeweils vertauscht
	VNC_fs.byte[1] = buffer[3];
	VNC_fs.byte[2] = buffer[0];
	VNC_fs.byte[3] = buffer[1];

	VNC_error = VNC_NO_ERR;
	}
UCSRB |= (1<<RXCIE);							// den RX_Interrupt wieder enablen	

} 

// -------------------------------------------------------------------------
// wartet auf den Eingang eines Bytes ber die serielle Schnittstelle
// dazu wird das Flag RXC gepollt - und VNC_timeout auf 0 geprft (= Timeout)
// wichtig: der RX_Interrupt muss disabled sein, sonst wird die Interrupt-Routine
// angesprungen und das Flag RXC durch Lesen von UDR gelscht,
// die Funktion wird somit nur durch einen Timeout-Fehler beendet !
void VNC_wait_for_rx(void)
{
VNC_timeout = VNC_PROMPT_WAIT;
while ((!(UCSRA & (1<<RXC))) && VNC_timeout);
}

// ----------------------------------------------------------------------------
// wird nach einem Befehl aufgerufen und wartet pollend auf das Prompt/SYNCH 
// oder den Fehlercode (zwei cHAR + RETURN = 3 Byte)
// wenn VNC_timeout == 0, dann wird beendet
// der Fehlercode steht in VNC_error: 		0 = kein Fehler

void VNC_wait_for_prompt(void)
{
uint8_t prompt[3];
uint8_t i;
VNC_error = VNC_ERR_TO;						// Timeout-Fehler

// den RX_Interrupt disablen, solange auf eine Antwort gewartet wird
UCSRB &= ~(1<<RXCIE);
													// warten auf 1. empfangenes Byte
VNC_wait_for_rx();							// RXC pollen, dabei Timeout ermglichen
													// das Lesen von UDR lscht RXC
if (!(VNC_timeout)) goto VNC_WAIT_EXIT;// Error

prompt[0] = UDR;								// 1. Byte schon mal speichern
													// zunchst der Test auf einen Erfolg
if ((prompt[0] == CHR_PROMPT) || (prompt[0] == CHR_SYNCH))
	{												// 2. Byte pollen
	VNC_wait_for_rx();						// RXC pollen, dabei Timeout ermglichen				
	if (VNC_timeout)							// wenn timeout > 0, dann noch kein Fehler
		{
		i = UDR;
		if (i == CHR_RETURN) VNC_error = 0; // Error zurcknehmen, bei Timeout bleibt error bestehen
		}
	goto VNC_WAIT_EXIT;
	}
													// wenn kein timeout, dann wird ein Fehler vorliegen,
													// der sollte mit 3 Byte dokumentiert werden, 
													// das letzte Byte ist RETURN
i = 1;											// das erste Byte ist bereits in VNC_error[0] abgelegt
while (i < 3)	
	{
	VNC_wait_for_rx();						// RXC pollen, dabei Timeout ermglichen
	prompt[i++] = UDR;
	if (!(VNC_timeout)) goto VNC_WAIT_EXIT;// exit
	}

// Die Fehlermeldung von 2 Byte + CR in einem Byte codieren:
// dazu die Summe der 3 Byte bilden, sie ist eindeutig und < 255  !
// in VNC_error ist im Erfolgsfall '0' oder Summe der ASCI-Werte der Fehlermeldung

VNC_error = prompt[0] + prompt[1] + prompt[2];

if 		(VNC_error == ('B' + 'C' + CHR_RETURN)) VNC_error = VNC_ERR_BC;
else if	(VNC_error == ('C' + 'F' + CHR_RETURN)) VNC_error = VNC_ERR_CF;
else if	(VNC_error == ('N' + 'D' + CHR_RETURN)) VNC_error = VNC_ERR_ND;
else if	(VNC_error == ('N' + 'E' + CHR_RETURN)) VNC_error = VNC_ERR_NE;
else if	(VNC_error == ('R' + 'O' + CHR_RETURN)) VNC_error = VNC_ERR_RO;
else if	(VNC_error == ('F' + 'I' + CHR_RETURN)) VNC_error = VNC_ERR_FI;
else if	(VNC_error == ('D' + 'F' + CHR_RETURN)) VNC_error = VNC_ERR_DF;
else if	(VNC_error == ('F' + 'O' + CHR_RETURN)) VNC_error = VNC_ERR_F0;
else	VNC_error = VNC_ERR;

VNC_WAIT_EXIT:

UCSRB |= (1<<RXCIE);							// den RX_Interrupt wieder enablen
return;
}

// ----------------------------------------------------------------------------
// wartet per Polling auf den Eingang der angeforderten Bytes
void VNC_wait_for_data(uint16_t cnt)
{
uint16_t i = 0;
uint8_t  j = 0;
VNC_error = VNC_ERR_TO;						// Timeout-Fehler als Vorgabe

// den RX_Interrupt disablen, solange wie auf eine Antwort gewartet wird
UCSRB &= ~(1<<RXCIE);

// solange einlesen, bis die Anzahl der Bytes erreicht ist oder ein Timeoverflow auftritt
while (i < cnt) 
	{
	VNC_wait_for_rx();						// RXC pollen, dabei Timeout ermglichen
	rx_buffer[i++] = UDR;
	if (!(VNC_timeout)) return;			// bei Timeout beenden, 
	}
													// noch zwei zustzliche Bytes empfangen:
VNC_wait_for_rx();							// Prompt und Return erwarten
j = UDR;
VNC_wait_for_rx();
j += UDR;

// wenn kein Timeoverflow und das vorletzte Zeichen ein Prompt, 
// und das letzte Zeichen ein RETRUN, dann liegt kein Fehler vor

if (VNC_timeout)
	{
	if (j == (CHR_PROMPT + CHR_RETURN)) VNC_error = VNC_NO_ERR;
	}

UCSRB |= (1<<RXCIE);							// den RX_Interrupt wieder enablen
return;
}

// ----------------------------------------------------------------------------
// Startet den Suspend-Modus
void VNC_suspend(void)
{
VNC_send_cmd(VNC_SUD);
}

// ----------------------------------------------------------------------------
// Beendet den Suspend-Modus
void VNC_wake_up(void)
{
VNC_send_cmd(VNC_WKD);
}

// ----------------------------------------------------------------------------
// variiert den Standard-Dateinamen durch einen Zhler (0-255) 
// die Ziffern werden aus den 3 Ziffern von filenumber bernommen
// -> FILE_xxx.TXT mit xxx = 000 bis 255
// -> DIR__xxx
void VNC_make_name(uint8_t filenumber, uint8_t typ)
{
char *pointer;
uint8_t i = 0;

if (typ == FILE)
	{
	pointer = &VNC_filename[0]; 
	}
else if (typ == DIR)
	{
	pointer = &VNC_dirname[0];
	}
else return;

while (filenumber > 99)
	{
	filenumber -= 100;
	i++;
	}
pointer[5] = ('0' + i);

i = 0;
while (filenumber > 9)
	{
	filenumber -= 10;
	i++;
	}
pointer[6] = ('0' + i);

i = 0;
while (filenumber > 0)
	{
	filenumber--;
	i++;
	}
pointer[7] = ('0' + i);
}


// --------------------------------------------------------------------------
// Schaltet den VNC wieder zurck auf 9600 Baud, so dass der mc per reset neu gestartet
// werden und den VNC ansprechen kann - ohne dass vor jedem RESET die Spannungsversorgung 
// am VNC gekappt werden muss.
void VNC_exit(void)
{
VNC_set_baudrate_9600();
//while(1);
}

//----------------------------------------------------------------------------
// springt an eine absolute Position innerhalb einer geffneten Datei (max 65KB)
// Fhrt zu Fehlermeldung bei leerer Datei !
void VNC_seek(uint16_t pos)
{
put_c(VNC_SEK);								// Instruction
put_c(CHR_SPACE);								// Space

put_c(0);										// Anzahl Bytes 1/4 (hchstwertiges Byte)
put_c(0);										// Anzahl Bytes 2/4
put_c((uint8_t) (pos/256));				// Anzahl Bytes 3/4
put_c((uint8_t) pos);						// Anzahl Bytes 4/4 (niedrigstwertiges Byte)
put_c(CHR_RETURN);							// dann noch ein RETURN senden
	
VNC_wait_for_prompt();	
}

// ----------------------------------------------------------------------------
// Kopiert die Datei 'source' auf die Datei 'target'.
// Danach wird blockweise aus der Quelldatei in das Array rx_buffer gelesen
// und anschlieend in die Zieldatei geschrieben.
// Die Zieldatei (falls sie existiert) wird zu Beginn gelscht (also berschrieben).

void VNC_copy_file(uint8_t source, uint8_t target)
{
uint16_t pointer = 0;
uint16_t fs;
VNC_make_name(source, FILE);

VNC_get_filesize();							// Dateigre bestimmen
if   (VNC_fs.word[0])  return;			// high-word > 1 -> Datei grer als 65 KB
if (!(VNC_fs.word[1])) return;			// low-word == 0 -> Datei leer
if (VNC_error) return;						// auch bei Fehler beenden (Datei fehlt z.B.)

fs = VNC_fs.word[1];							// Filesize nach fs

VNC_make_name(target, FILE);				// den Namen der Zieldatei erzeugen
VNC_file_access(FILE_CLOSE);				// prophylaktisch schlieen
VNC_file_access(FILE_DELETE);				// Zieldatei lschen, sofern sie existiert
													// keine Fehlermeldung prfen
while (fs)
	{
	VNC_make_name(source, FILE);			// den Namen der Quelldatei erzeugen
	VNC_file_access(FILE_READ);			// Quelldatei zu lesen ffnen
	VNC_seek(pointer);
	
	if (fs >= BLOCK_SIZE)					// Blockweise einlesen
		{
		VNC_read_data(BLOCK_SIZE);			// Daten von source in rx_buffer kopieren
		}

	else
		{
		VNC_read_data(fs);					// letzter Rest der Datei
		}

	VNC_file_access(FILE_CLOSE);			// Datei schlieen		
	if (VNC_error) return;
	
	VNC_make_name(target, FILE);			// den Namen der Zieldatei erzeugen
	VNC_file_access(FILE_WRITE);			// Zieldatei ffnen

	if (fs >= BLOCK_SIZE)					// Blockweise schreiben
		{
		VNC_write_data(BLOCK_SIZE, rx_buffer);	// rx_buffer senden
		fs -= BLOCK_SIZE;						// Filesize decrementieren
		pointer += BLOCK_SIZE;				// den Pointer incrementieren
		}
		
	else
		{
		VNC_write_data(fs, rx_buffer);	// den Rest schreiben
		fs = 0;									// Filesize auf 0 
		}
	
	VNC_file_access(FILE_CLOSE);	
	if (VNC_error) return;					// Bei Fehler auch vorzeitig abbrechen
	} 
} 

// ----------------------------------------------------------------------------
// Wird aufgerufen bei der Fehlermeldung 'ND' (No Disk).
// Wartet, bis eine Disk eingelegt wird und setzt dann den Programmablauf fort.
// Beim Anstecken wird gesendet: 'DD2' + CR 'NU'+ CR + CHR_Prompt + CR.
// Wenn die RX_ISR ein CR empfngt, dann setzt sie das Flag isr_flag.
// Es mssen drei CR empfangen werden, wenn der VNC ein disk insert meldet.

// vereinfachte Lsung:
// wenn der VNC eine Message absetzt, dann wird kurz gewartet und anschlieend der Stick
// mit EMPTY COMMAND abgefragt: gibt es keine Fehlermeldung, dann ist er prsent.
void VNC_insert_disk(void)
{
isr_flag = 0;									// Flag lschen

while(1)			
	{
	VNC_timeout = 5;							// zum Togglen der LED
	while(VNC_timeout && (!(isr_flag)));// Zeit und rx_flag testen
	LEDPORT ^= (1<<LED_READY);				// nach Ablauf der Zeit LED togglen

	if (isr_flag)								// wenn Daten eingegangen sind
		{
		isr_flag = 0;							// Flag zurcknehmen
		VNC_timeout = 20;						// Wartezeit 20 * 33ms, 
		while(VNC_timeout);					// damit der Stick seine Meldung absetzen kann	
		
		VNC_synch();
		VNC_send_cmd(VNC_EMPTY_CMD);		// Testen, ob ein stick gefunden
		if (!(VNC_error)) break; 			// wenn ja, dann beenden
		}
	}
} 

// ----------------------------------------------------------------------------
// wartet darauf, dass der Stick entnommen und anschlieend wieder angesteckt wird.
// Zunchst wird gewartet, bis der Stick abgezogen wird - oder ein Flag die Aktion beendet.
// Beim Entfernen der Disk wird gesendet: DR2 + CR + ND + CR
// Wenn die RX_ISR ein CR empfngt, dann setzt sie das Flag isr_flag
// es mssen also zwei CR empfangen werden, wenn der VNC ein disk remove meldet
// mit einem EMPTY COMMAND wird anschlieend geprft, ob tatschlich ein ND zurckgegeben wird.

// vereinfachte Lsung:
// wenn der VNC eine Sendung absetzt, dann wird geprft, ob ein Stick antwortet.
// - wenn nicht, dann wird in VNC_insert_disk() verzweigt.
// - wenn doch, dann wird beendet.
void VNC_remove_disk(void)
{
isr_flag = 0;
VNC_set_baudrate_9600();					// zurckschalten auf default 9600, 
													// fr den Fall, dass ein reset am mc ausgefhrt wird
LEDPORT |= (1<<LED_READY);					// LED ausschalten
LEDPORT ^= (1<<LED_WAIT);

while(1)
	{
	while(!(isr_flag));						// warten, bis Daten ber die serielle Schnittstelle kommen

	isr_flag = 0;								// Flag zurcknehmen

	VNC_timeout = 20;							// Wartezeit
	while(VNC_timeout);
	
	VNC_send_cmd(VNC_EMPTY_CMD);			// Disk testen
	if (VNC_error) 							// wenn NoDisk, dann wurde die Disk abgezogen
		{
		PORTD |= (1<<LED_WAIT);
		VNC_insert_disk();					// auf das neue Anstpseln des Stick warten ...
		}


	VNC_send_cmd(VNC_EMPTY_CMD);			// Disk testen nach Rckkehr aus VNC_insert_disk()

	if (!(VNC_error)) 						// wenn kein Fehler, dann ist der Stick gefunden
		{
		PORTD &= ~(1<<LED_READY);			// LED wieder an
		#if VNC_BAUD_RATE == 19200	
		VNC_set_baudrate_19200(); 			// Baudrate des VNC von default 9600 auf x ndern
		#elif VNC_BAUD_RATE == 38400	
		VNC_set_baudrate_38400(); 
		#elif VNC_BAUD_RATE == 57200	
		VNC_set_baudrate_38400(); 
		#elif VNC_BAUD_RATE == 115200
		VNC_set_baudrate_115200();	
		#endif
		
		VNC_synch();
		break;
		}
	}
} 

// ---------------------------------------------------------------------------
// bergibt einen Filenamen an eine lokale Variable
/*
void VNC_set_filename(char *file)
{
uint8_t i = 0;

while (file[i])
	{
	VNC_filename[i] = file[i];
	i++;
	} 
VNC_filename[i] = 0;
} */


// --- [ eof ] ----------------------------------------------------------------